@@ -6,16 +6,17 @@
  * Author:
  *   Nat Friedman (nat@ximian.com)
  *
- * Copyright 2000, Ximian, Inc.
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  */
 
 #ifndef __E_BOOK_LISTENER_H__
 #define __E_BOOK_LISTENER_H__
 
 #include <bonobo/bonobo-object.h>
-#include <libebook/Evolution-DataServer-Addressbook.h>
 #include <libebook/e-book-types.h>
 
+#include "Evolution-DataServer-Addressbook.h"
+
 #define E_TYPE_BOOK_LISTENER        (e_book_listener_get_type ())
 #define E_BOOK_LISTENER(o)          (G_TYPE_CHECK_INSTANCE_CAST ((o), E_TYPE_BOOK_LISTENER, EBookListener))
 #define E_BOOK_LISTENER_CLASS(k)    (G_TYPE_CHECK_CLASS_CAST((k), E_TYPE_BOOK_LISTENER, EBookListenerClass))
