@@ -326,8 +326,6 @@ e_cal_backend_groupwise_set_attachments_from_comp (ECalComponent *comp,
 			continue;
 		}
 
-		g_free (attach_filename_full);
-
 		attach_item = g_new0 (EGwItemAttachment, 1);
 		/* FIXME the member does not follow the naming convention.
 		 * Should be fixed in e-gw-item*/
@@ -340,6 +338,7 @@ e_cal_backend_groupwise_set_attachments_from_comp (ECalComponent *comp,
 		attach_item->data = encoded_data;
 		attach_item->size = strlen (encoded_data);
 
+		g_free (attach_filename_full);
 		g_free (file_contents);
 		attach_list = g_slist_append (attach_list, attach_item);
 	}
@@ -1086,18 +1085,13 @@ e_gw_item_to_cal_component (EGwItem *item, ECalBackendGroupwise *cbgw)
 	t = e_gw_item_get_creation_date (item);
 	if (t) {
 		itt_utc = icaltime_from_string (t);
-		if (!icaltime_get_timezone (itt_utc))
-			icaltime_set_timezone (&itt_utc, icaltimezone_get_utc_timezone());
-		if (default_zone) {
-			itt = icaltime_convert_to_zone (itt_utc, default_zone);
-			icaltime_set_timezone (&itt, default_zone);
-			e_cal_component_set_created (comp, &itt);
-			e_cal_component_set_dtstamp (comp, &itt);
 
-		} else {
-			e_cal_component_set_created (comp, &itt_utc);
-			e_cal_component_set_dtstamp (comp, &itt_utc);
-		}
+		/* RFC 2445 - CREATED/DTSTAMP/LAST-MODIFIED always in UTC */
+		icaltimezone_convert_time (&itt_utc, (icaltimezone*) icaltime_get_timezone (itt_utc), icaltimezone_get_utc_timezone ());
+		icaltime_set_timezone (&itt_utc, icaltimezone_get_utc_timezone ());
+
+		e_cal_component_set_created (comp, &itt_utc);
+		e_cal_component_set_dtstamp (comp, &itt_utc);
 	}
 
 	/* categories */
@@ -1592,7 +1586,7 @@ e_gw_connection_create_appointment (EGwConnection *cnc, const char *container, E
 
 static EGwConnectionStatus
 start_freebusy_session (EGwConnection *cnc, GList *users,
-               time_t start, time_t end, const char **session)
+               time_t start, time_t end, char **session)
 {
         SoupSoapMessage *msg;
         SoupSoapResponse *response;
