@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000 Ximian Inc.
+ *  Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  *  Authors: Michael Zucchi <notzed@ximian.com>
  *           Jeffrey Stedfast <fejj@ximian.com>
@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifndef _CAMEL_FILTER_DRIVER_H
@@ -54,7 +54,7 @@ enum camel_filter_status_t {
 	CAMEL_FILTER_STATUS_START,	/* start of new message processed */
 	CAMEL_FILTER_STATUS_ACTION,	/* an action performed */
 	CAMEL_FILTER_STATUS_PROGRESS,	/* (an) extra update(s), if its taking longer to process */
-	CAMEL_FILTER_STATUS_END,	/* end of message */
+	CAMEL_FILTER_STATUS_END		/* end of message */
 };
 
 typedef CamelFolder * (*CamelFilterGetFolderFunc) (CamelFilterDriver *driver, const char *uri,
@@ -90,7 +90,7 @@ int  camel_filter_driver_remove_rule_by_name  (CamelFilterDriver *d, const char
 void camel_filter_driver_flush                (CamelFilterDriver *driver, CamelException *ex);
 
 int  camel_filter_driver_filter_message       (CamelFilterDriver *driver, CamelMimeMessage *message,
-					       CamelMessageInfo *info, const char *uri,
+					       CamelMessageInfo *info, const char *uid,
 					       CamelFolder *source, const char *source_url,
 					       const char *original_source_url, CamelException *ex);
 
