@@ -34,6 +34,7 @@
 #include <glib/gi18n-lib.h>
 #include <glib/gstdio.h>
 
+#include "camel-db.h"
 #include "camel-file-utils.h"
 #include "camel-mime-message.h"
 #include "camel-stream-null.h"
@@ -64,6 +65,7 @@ static int local_summary_load(CamelLocalSummary *cls, int forceindex, CamelExcep
 static int local_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changeinfo, CamelException *ex);
 static int local_summary_sync(CamelLocalSummary *cls, gboolean expunge, CamelFolderChangeInfo *changeinfo, CamelException *ex);
 static CamelMessageInfo *local_summary_add(CamelLocalSummary *cls, CamelMimeMessage *msg, const CamelMessageInfo *info, CamelFolderChangeInfo *, CamelException *ex);
+static int local_summary_need_index(void);
 
 static void camel_local_summary_class_init (CamelLocalSummaryClass *klass);
 static void camel_local_summary_init       (CamelLocalSummary *obj);
@@ -110,6 +112,7 @@ camel_local_summary_class_init(CamelLocalSummaryClass *klass)
 
 	klass->encode_x_evolution = local_summary_encode_x_evolution;
 	klass->decode_x_evolution = local_summary_decode_x_evolution;
+	klass->need_index = local_summary_need_index;
 }
 
 static void
@@ -159,7 +162,7 @@ camel_local_summary_load(CamelLocalSummary *cls, int forceindex, CamelException
 {
 	d(printf("Loading summary ...\n"));
 
-	if (forceindex
+	if ((forceindex && ((CamelLocalSummaryClass *)(CAMEL_OBJECT_GET_CLASS(cls)))->need_index())
 	    || ((CamelLocalSummaryClass *)(CAMEL_OBJECT_GET_CLASS(cls)))->load(cls, forceindex, ex) == -1) {
 		w(g_warning("Could not load summary: flags may be reset"));
 		camel_folder_summary_clear((CamelFolderSummary *)cls);
@@ -421,10 +424,50 @@ local_summary_sync(CamelLocalSummary *cls, gboolean expunge, CamelFolderChangeIn
 	return ret;
 }
 
+static int
+local_summary_need_index(void) {
+	return 1;
+}
+
+static void
+update_summary (CamelFolderSummary *summary, CamelMessageInfoBase *info, CamelMessageInfoBase *old)
+{
+	int unread=0, deleted=0, junk=0;
+	guint32 flags = info->flags;
+	guint32 oldflags = old->flags;
+
+	if ((flags & CAMEL_MESSAGE_SEEN) != (oldflags & CAMEL_MESSAGE_SEEN))
+		unread = (oldflags & CAMEL_MESSAGE_SEEN) ? 1 : -1;
+	
+	if ((flags & CAMEL_MESSAGE_DELETED) != (oldflags & CAMEL_MESSAGE_DELETED))
+		deleted = (oldflags & CAMEL_MESSAGE_DELETED) ? 1 : -1;
+
+	if ((flags & CAMEL_MESSAGE_JUNK) != (oldflags & CAMEL_MESSAGE_JUNK))
+		junk = (oldflags & CAMEL_MESSAGE_JUNK) ? 1 : -1;
+	
+	/* Things would already be flagged */
+	
+	if (summary) {
+		
+		if (unread)
+			summary->unread_count -= unread;
+		if (deleted)
+			summary->deleted_count += deleted;
+		if (junk)
+			summary->junk_count += junk;
+		if (junk && !deleted)
+			summary->junk_not_deleted_count += junk;
+		if (junk ||  deleted) 
+			summary->visible_count -= junk ? junk : deleted;
+	}
+
+}
+
 static CamelMessageInfo *
 local_summary_add(CamelLocalSummary *cls, CamelMimeMessage *msg, const CamelMessageInfo *info, CamelFolderChangeInfo *ci, CamelException *ex)
 {
 	CamelLocalMessageInfo *mi;
+	CamelFolderSummary *s = (CamelFolderSummary *)cls;
 	char *xev;
 
 	d(printf("Adding message to summary\n"));
@@ -445,8 +488,10 @@ local_summary_add(CamelLocalSummary *cls, CamelMimeMessage *msg, const CamelMess
 				camel_message_info_set_user_tag((CamelMessageInfo *)mi, tag->name, tag->value);
 				tag = tag->next;
 			}
-
+			
+			update_summary (s, (CamelMessageInfoBase *) mi, (CamelMessageInfoBase *) info);
 			mi->info.flags |= (camel_message_info_flags(info) & 0xffff);
+			camel_folder_summary_update_flag_cache (s, mi->info.uid, mi->info.flags);
 			mi->info.size = camel_message_info_size(info);
 		}
 
@@ -597,7 +642,7 @@ static int
 summary_header_from_db (CamelFolderSummary *s, CamelFIRecord *fir)
 {
 	CamelLocalSummary *cls = (CamelLocalSummary *)s;
-	char *part;
+	char *part, *tmp;
 
 	/* We dont actually add our own headers, but version that we don't anyway */
 
@@ -608,7 +653,11 @@ summary_header_from_db (CamelFolderSummary *s, CamelFIRecord *fir)
 	if (part) {
 		EXTRACT_FIRST_DIGIT (cls->version)
 	}
-	fir->bdata = part;
+
+	/* keep only the rest of the bdata there (strip our version digit) */
+	tmp = g_strdup (part);
+	g_free (fir->bdata);
+	fir->bdata = tmp;
 
 	return 0;
 }
